@@ -11,15 +11,10 @@
 #include "topo.hpp"      // for Topo
 #include "chemcomp.hpp"  // for ChemComp
 #include "calculate.hpp" // for calculate_angle
+#include "modify.hpp"    // for remove_hydrogens
 
 namespace gemmi {
 
-struct BondedAtom {
-  Atom* ptr;
-  Position& pos; // == ptr->pos;
-  double dist;
-};
-
 // Assumes no hydrogens in the residue.
 // Position and serial number are not assigned for new atoms.
 inline void add_hydrogens_without_positions(const ChemComp& cc, Residue& res) {
@@ -126,31 +121,35 @@ position_from_two_angles(const Position& p1,
 }
 
 
-inline void place_hydrogens(const Atom& atom, Topo::ResInfo& ri,
-                            const Topo& topo) {
+inline void place_hydrogens(const Topo& topo, const Atom& atom) {
   using Angle = Restraints::Angle;
+  struct BondedAtom {
+    Atom* ptr;
+    Position& pos; // == ptr->pos;
+    double dist;
+  };
 
   // put atoms bonded to atom into two lists
   std::vector<BondedAtom> known; // heavy atoms with known positions
   std::vector<BondedAtom> hs;    // H atoms (unknown)
-  for (const Topo::Rule& rule : ri.rules)
-    if (rule.rkind == Topo::RKind::Bond) {
-      const Topo::Bond& t = topo.bonds[rule.index];
-      int n = Topo::has_atom(&atom, t);
-      if (n == 0 || n == 1) {
-        Atom* other = t.atoms[1-n];
-        if (other->altloc) {
-          if (atom.altloc && atom.altloc != other->altloc)
-            continue;
-          if (atom.altloc == '\0' &&
-              in_vector_f([&](const BondedAtom& a) { return a.ptr->name == other->name; },
-                          known))
-            continue;
-        }
-        auto& atom_list = other->is_hydrogen() ? hs : known;
-        atom_list.push_back({other, other->pos, t.restr->value});
-      }
+  known.reserve(3);
+  hs.reserve(4);
+
+  auto range = topo.bond_index.equal_range(&atom);
+  for (auto i = range.first; i != range.second; ++i) {
+    const Topo::Bond* t = i->second;
+    Atom* other = t->atoms[t->atoms[0] == &atom ? 1 : 0];
+    if (other->altloc) {
+      if (atom.altloc && atom.altloc != other->altloc)
+        continue;
+      if (atom.altloc == '\0' &&
+          in_vector_f([&](const BondedAtom& a) { return a.ptr->name == other->name; },
+                      known))
+        continue;
     }
+    auto& atom_list = other->is_hydrogen() ? hs : known;
+    atom_list.push_back({other, other->pos, t->restr->value});
+  }
 
   if (hs.size() == 0)
     return;
@@ -215,8 +214,8 @@ inline void place_hydrogens(const Atom& atom, Topo::ResInfo& ri,
     int period = 0;
     const Atom* tau_end = nullptr;
     for (const Topo::Plane& plane : topo.planes) {
-      if (plane.atoms.size() > 3 &&
-          plane.has(h.ptr) && plane.has(&atom) && plane.has(heavy.ptr)) {
+      // only Topo::Plane with atoms.size() >= 4 is put into planes
+      if (plane.has(h.ptr) && plane.has(&atom) && plane.has(heavy.ptr)) {
         for (const Atom* a : plane.atoms) {
           if (!a->is_hydrogen() && a != &atom && a != heavy.ptr) {
             tau_end = a;
@@ -346,37 +345,51 @@ inline void place_hydrogens(const Atom& atom, Topo::ResInfo& ri,
   }
 }
 
-inline void adjust_hydrogen_distances(Topo& topo, Restraints::DistanceOf of, double default_scale=1.) {
+inline void adjust_hydrogen_distances(Topo& topo, Restraints::DistanceOf of,
+                                      double default_scale=1.) {
+  for (const Topo::Bond& t : topo.bonds) {
+    assert(t.atoms[0] != nullptr && t.atoms[1] != nullptr);
+    if (t.atoms[0]->is_hydrogen() || t.atoms[1]->is_hydrogen()) {
+      Position u = t.atoms[1]->pos - t.atoms[0]->pos;
+      double scale = t.restr->distance(of) / u.length();
+      if (std::isnan(scale))
+        scale = default_scale;
+      if (t.atoms[1]->is_hydrogen())
+        t.atoms[1]->pos = t.atoms[0]->pos + u * scale;
+      else
+        t.atoms[0]->pos = t.atoms[1]->pos - u * scale;
+    }
+  }
+}
+
+inline void place_hydrogens_on_all_atoms(Topo& topo) {
   for (Topo::ChainInfo& chain_info : topo.chain_infos)
     for (Topo::ResInfo& ri : chain_info.res_infos)
-      for (const Topo::Rule& rule : ri.rules)
-        if (rule.rkind == Topo::RKind::Bond) {
-          const Topo::Bond& t = topo.bonds[rule.index];
-          assert(t.atoms[0] != nullptr && t.atoms[1] != nullptr);
-          if (t.atoms[0]->is_hydrogen() || t.atoms[1]->is_hydrogen()) {
-            Position u = t.atoms[1]->pos - t.atoms[0]->pos;
-            double scale = t.restr->distance(of) / u.length();
-            if (std::isnan(scale)) scale = default_scale;
-            if (t.atoms[1]->is_hydrogen())
-              t.atoms[1]->pos = t.atoms[0]->pos + u * scale;
-            else
-              t.atoms[0]->pos = t.atoms[1]->pos - u * scale;
+      for (Atom& atom : ri.res->atoms)
+        if (!atom.is_hydrogen()) {
+          try {
+            place_hydrogens(topo, atom);
+          } catch (const std::runtime_error& e) {
+            topo.err("Placing of hydrogen bonded to "
+                     + atom_str(chain_info.chain_ref, *ri.res, atom)
+                     + " failed:\n  " + e.what());
           }
         }
 }
 
-enum class HydrogenChange { None, Shift, Remove, ReAdd, ReAddButWater };
+enum class HydrogenChange { NoChange, Shift, Remove, ReAdd, ReAddButWater };
 
 inline std::unique_ptr<Topo>
 prepare_topology(Structure& st, MonLib& monlib, size_t model_index,
-                 HydrogenChange h_change, bool reorder, bool raise_errors=false,
-                 bool ignore_unknown_links=false) {
+                 HydrogenChange h_change, bool reorder,
+                 std::ostream* warnings=nullptr, bool ignore_unknown_links=false) {
   std::unique_ptr<Topo> topo(new Topo);
+  topo->warnings = warnings;
   if (model_index >= st.models.size())
     fail("no such model index: " + std::to_string(model_index));
   topo->initialize_refmac_topology(st, st.models[model_index], monlib, ignore_unknown_links);
 
-  bool keep = (h_change == HydrogenChange::None || h_change == HydrogenChange::Shift);
+  bool keep = (h_change == HydrogenChange::NoChange || h_change == HydrogenChange::Shift);
   if (!keep || reorder) {
     // remove/add hydrogens, sort atoms, set sequential serial numbers
     int serial = 0;
@@ -394,7 +407,7 @@ prepare_topology(Structure& st, MonLib& monlib, size_t model_index,
           for (Atom& atom : res.atoms) {
             auto it = cc.find_atom(atom.name);
             if (it == cc.atoms.end())
-              fail("No atom ", atom.name, " expected in ", res.name);
+              topo->err("no atom " + atom.name + " expected in " + res.name);
             atom.serial = int(it - cc.atoms.begin()); // temporary, for sorting only
           }
           std::sort(res.atoms.begin(), res.atoms.end(), [](const Atom& a, const Atom& b) {
@@ -410,23 +423,8 @@ prepare_topology(Structure& st, MonLib& monlib, size_t model_index,
   topo->finalize_refmac_topology(monlib);
 
   // the hydrogens added previously have positions not set
-  if (h_change != HydrogenChange::None) {
-    for (Topo::ChainInfo& chain_info : topo->chain_infos)
-      for (Topo::ResInfo& ri : chain_info.res_infos)
-        for (Atom& atom : ri.res->atoms)
-          if (!atom.is_hydrogen()) {
-            try {
-              place_hydrogens(atom, ri, *topo);
-            } catch (const std::runtime_error& e) {
-              std::string err = "Placing of hydrogen bonded to " +
-                                atom_str(chain_info.name, *ri.res, atom.name, atom.altloc) +
-                                " failed:\n  " + e.what();
-              if (raise_errors)
-                fail(err);
-              std::fprintf(stderr, "%s\n", err.c_str());
-            }
-          }
-  }
+  if (h_change != HydrogenChange::NoChange)
+    place_hydrogens_on_all_atoms(*topo);
 
   return topo;
 }
