@@ -569,7 +569,7 @@ struct GroupOps {
   std::array<int, 3> find_grid_factors() const {
     const int T = Op::DEN;
     int r[3] = {T, T, T};
-    for (const Op& op : *this)
+    for (Op op : *this)
       for (int i = 0; i != 3; ++i)
         if (op.tran[i] != 0 && op.tran[i] < r[i])
           r[i] = op.tran[i];
@@ -1113,13 +1113,13 @@ namespace impl {
 template<class Dummy>
 struct Tables_
 {
-  static const SpaceGroup main[555];
+  static const SpaceGroup main[557];
   static const SpaceGroupAltName alt_names[28];
   static const unsigned char ccp4_hkl_asu[230];
 };
 
 template<class Dummy>
-const SpaceGroup Tables_<Dummy>::main[555] = {
+const SpaceGroup Tables_<Dummy>::main[557] = {
   // This table was generated by tools/gen_sg_table.py.
   // First 530 entries in the same order as in SgInfo, sgtbx and ITB.
   // Note: spacegroup 68 has three duplicates with different H-M names.
@@ -1684,6 +1684,8 @@ const SpaceGroup Tables_<Dummy>::main[555] = {
   {117,    0, "C -4 2 b"  ,   0,     "", "C -4 2ya"      , 48}, // 552
   { 97,    0, "F 4 2 2" ,     0,     "", "F 4 2"         , 48}, // 553
   {139,    0, "F 4/m m m" ,   0,     "", "-F 4 2"        , 48}, // 554
+  { 89,    0, "C 4 2 2" ,     0,     "", "C 4 2"         , 48}, // 555
+  { 90,    0, "C 4 2 21" ,    0,     "", "C 4a 2"        , 48}, // 556
 };
 
 template<class Dummy>
