@@ -7,6 +7,7 @@
 #include "gemmi/fprime.hpp"
 #include "gemmi/reciproc.hpp" // for count_reflections, make_miller_vector
 #include "gemmi/cif2mtz.hpp"  // for CifToMtz
+#include "gemmi/merge.hpp"  // for Intensities
 
 #include "common.h"
 #include "arrvec.h"  // py_array_from_vector
@@ -160,4 +161,77 @@ void add_hkl(py::module& m) {
         return new Mtz(self.convert_block_to_mtz(rb, out));
     })
     ;
+
+  py::class_<Intensities> intensities(m, "Intensities");
+
+  py::enum_<Intensities::Type>(intensities, "Type")
+    .value("Unknown", Intensities::Type::Unknown)
+    .value("Unmerged", Intensities::Type::Unmerged)
+    .value("Mean", Intensities::Type::Mean)
+    .value("Anomalous", Intensities::Type::Anomalous)
+    ;
+
+  intensities
+    .def(py::init<>())
+    .def_readwrite("spacegroup", &Intensities::spacegroup)
+    .def_readwrite("unit_cell", &Intensities::unit_cell)
+    .def_readwrite("type", &Intensities::type)
+    .def("resolution_range", &Intensities::resolution_range)
+    .def("remove_systematic_absences", &Intensities::remove_systematic_absences)
+    .def("merge_in_place", &Intensities::merge_in_place, py::arg("itype"))
+    .def("read_mtz", &Intensities::read_mtz, py::arg("mtz"), py::arg("type"))
+    .def_property_readonly("miller_array", [](const Intensities& self) {
+      const Intensities::Refl* data = self.data.data();
+      py::array::ShapeContainer shape({(py::ssize_t)self.data.size(), 3});
+      py::array::StridesContainer strides({(const char*)(data+1) - (const char*)data,
+                                           sizeof(int)});
+      return py::array_t<int>(shape, strides, &data->hkl[0], py::cast(self));
+    }, py::return_value_policy::reference_internal)
+    .def_property_readonly("value_array", [](const Intensities& self) {
+      const Intensities::Refl* data = self.data.data();
+      py::ssize_t stride = (const char*)(data+1) - (const char*)data;
+      return py::array_t<double>({(py::ssize_t)self.data.size()}, {stride},
+                                 &data->value, py::cast(self));
+    }, py::return_value_policy::reference_internal)
+    .def_property_readonly("sigma_array", [](const Intensities& self) {
+      const Intensities::Refl* data = self.data.data();
+      py::ssize_t stride = (const char*)(data+1) - (const char*)data;
+      return py::array_t<double>({(py::ssize_t)self.data.size()}, {stride},
+                                 &data->sigma, py::cast(self));
+    }, py::return_value_policy::reference_internal)
+    .def_property_readonly("nobs_array", [](const Intensities& self) {
+      const Intensities::Refl* data = self.data.data();
+      py::ssize_t stride = (const char*)(data+1) - (const char*)data;
+      return py::array_t<short>({(py::ssize_t)self.data.size()}, {stride},
+                                &data->nobs, py::cast(self));
+    }, py::return_value_policy::reference_internal)
+    .def_property_readonly("isign_array", [](const Intensities& self) {
+      const Intensities::Refl* data = self.data.data();
+      py::ssize_t stride = (const char*)(data+1) - (const char*)data;
+      return py::array_t<short>({(py::ssize_t)self.data.size()}, {stride},
+                                &data->isign, py::cast(self));
+    }, py::return_value_policy::reference_internal)
+    .def("set_data", [](Intensities& self, const UnitCell& unit_cell, 
+                        const SpaceGroup* sg, py::array_t<int> hkl, 
+                        py::array_t<double> values, py::array_t<double> sigmas) {
+      auto h = hkl.unchecked<2>();
+      if (h.shape(1) != 3)
+        throw std::domain_error("error: the size of the second dimension != 3");
+      auto v = values.template unchecked<1>();
+      auto s = sigmas.template unchecked<1>();
+      if (h.shape(0) != v.shape(0) || h.shape(0) != s.shape(0))
+        throw std::domain_error("error: arrays have different lengths");
+
+      self.unit_cell = unit_cell;
+      self.spacegroup = sg;
+      self.data.reserve(h.shape(0));
+      for (py::ssize_t i = 0; i < h.shape(0); ++i)
+        if (!std::isnan(v(i)) && s(i) > 0)
+          self.data.push_back({{{h(i, 0), h(i, 1), h(i, 2)}}, 1, 0, v(i), s(i)});
+
+      self.switch_to_asu_indices();
+      self.type = Intensities::Type::Unmerged;
+    }, py::arg("cell"), py::arg("sg").none(false),
+       py::arg("miller_array"), py::arg("value_array"), py::arg("sigma_array"))
+    ;
 }
