@@ -68,6 +68,7 @@ void add_monlib(py::module& m) {
 
   py::class_<MonLib>(m, "MonLib")
     .def(py::init<>())
+    .def_readonly("mon_lib_list", &MonLib::mon_lib_list)
     .def_readonly("monomers", &MonLib::monomers)
     .def_readonly("links", &MonLib::links)
     .def_readonly("modifications", &MonLib::modifications)
@@ -93,6 +94,7 @@ void add_monlib(py::module& m) {
     .def("insert_comp_list", [](MonLib &self, const cif::Document &doc) {
         insert_comp_list(doc, self.residue_infos);
     })
+    .def("path", &MonLib::path, py::arg("code")=nullptr)
     .def("__repr__", [](const MonLib& self) {
         return "<gemmi.MonLib with " +
                std::to_string(self.monomers.size()) + " monomers, " +
@@ -101,9 +103,10 @@ void add_monlib(py::module& m) {
     });
 
   m.def("read_monomer_lib", [](const std::string& monomer_dir,
-                               const std::vector<std::string>& resnames) {
-    return read_monomer_lib(monomer_dir, resnames, gemmi::read_cif_gz);
-  });
+                               const std::vector<std::string>& resnames,
+                               bool ignore_missing) {
+    return read_monomer_lib(monomer_dir, resnames, gemmi::read_cif_gz, ignore_missing);
+  }, py::arg("monomer_dir"), py::arg("resnames"), py::arg("ignore_missing")=false);
   m.def("read_monomer_cif", [](const std::string& path) {
     return read_monomer_cif(path, gemmi::read_cif_gz);
   });
