@@ -1,8 +1,9 @@
 // Copyright 2017 Global Phasing Ltd.
 
 #include <stdio.h>
-#include <cstdlib> // for getenv
+#include <cstdlib>   // for getenv
 #include <algorithm> // for count_if
+#include <iostream>  // for cerr
 #include <stdexcept>
 #include "gemmi/chemcomp.hpp"  // for ChemComp
 #include "gemmi/to_cif.hpp"    // for write_cif_to_stream
@@ -11,7 +12,8 @@
 #include "gemmi/to_mmcif.hpp"  // for write_struct_conn
 #include "gemmi/sprintf.hpp"   // for to_str, to_str_prec
 #include "gemmi/calculate.hpp" // for find_best_plane
-#include "gemmi/polyheur.hpp"  // for remove_hydrogens
+#include "gemmi/modify.hpp"    // for remove_hydrogens
+#include "gemmi/polyheur.hpp"  // for setup_entities
 #include "gemmi/monlib.hpp"    // for MonLib, read_monomer_lib
 #include "gemmi/topo.hpp"      // for Topo
 #include "gemmi/placeh.hpp"    // for place_hydrogens
@@ -50,12 +52,6 @@ const option::Descriptor Usage[] = {
 };
 
 
-// Topology: restraints applied to a model
-int count_provenance(const std::vector<Topo::Rule>& rules, Topo::Provenance p) {
-  return std::count_if(rules.begin(), rules.end(),
-                       [&](const Topo::Rule& r) { return r.provenance == p; });
-}
-
 bool has_anisou(const gemmi::Model& model) {
   for (const gemmi::Chain& chain : model.chains)
     for (const gemmi::Residue& res : chain.residues)
@@ -74,9 +70,7 @@ std::string get_ccp4_mod_id(const std::vector<std::string>& mods) {
   return ".";
 }
 
-cif::Document make_crd(const gemmi::Structure& st,
-                       const gemmi::MonLib& monlib,
-                       const Topo& topo) {
+cif::Document make_crd(const gemmi::Structure& st, const Topo& topo) {
   using gemmi::to_str;
   cif::Document crd;
   auto e_id = st.info.find("_entry.id");
@@ -115,12 +109,12 @@ cif::Document make_crd(const gemmi::Structure& st,
     if (!chain_info.polymer)
       continue;
     for (const Topo::ResInfo& ri : chain_info.res_infos) {
-      const Topo::ResInfo::Prev* prev = ri.prev.empty() ? nullptr : &ri.prev[0];
+      const Topo::Link* prev = ri.prev.empty() ? nullptr : &ri.prev[0];
       poly_loop.add_row({ri.res->name,
                          ri.res->seqid.str(),
                          chain_info.entity_id,
-                         prev ? prev->link : ".",
-                         prev ? prev->get(&ri)->res->seqid.str() : "n/a",
+                         prev ? prev->link_id : ".",
+                         prev ? prev->res1->seqid.str() : "n/a",
                          get_ccp4_mod_id(ri.mods)});
     }
   }
@@ -208,18 +202,21 @@ cif::Document make_crd(const gemmi::Structure& st,
       atom_loop.tags.push_back(std::string("_atom_site.aniso_U") + idx);
   std::vector<std::string>& vv = atom_loop.values;
   vv.reserve(count_atom_sites(st) * atom_loop.tags.size());
-  for (const gemmi::Chain& chain : model0.chains) {
-    for (const gemmi::Residue& res : chain.residues) {
+
+
+  for (const Topo::ChainInfo& chain_info : topo.chain_infos)
+    for (const Topo::ResInfo& ri : chain_info.res_infos) {
+      const gemmi::ChemComp& cc = ri.chemcomp;
+      const gemmi::Residue& res = *ri.res;
       std::string auth_seq_id = res.seqid.num.str();
       //std::string ins_code(1, res.icode != ' ' ? res.icode : '?');
-      const gemmi::ChemComp& cc = monlib.monomers.at(res.name);
       for (const gemmi::Atom& a : res.atoms) {
         vv.emplace_back("ATOM");
         vv.emplace_back(std::to_string(a.serial));
         vv.emplace_back(a.name);
         vv.emplace_back(1, a.altloc ? a.altloc : '.');
         vv.emplace_back(res.name);
-        vv.emplace_back(cif::quote(chain.name));
+        vv.emplace_back(cif::quote(chain_info.chain_ref.name));
         vv.emplace_back(auth_seq_id);
         //vv.emplace_back(ins_code);
         vv.emplace_back(to_str(a.pos.x));
@@ -243,12 +240,10 @@ cif::Document make_crd(const gemmi::Structure& st,
         }
       }
     }
-  }
   return crd;
 }
 
-void add_restraints(const Topo::Rule rule,
-                    const Topo& topo, const Restraints& rt,
+void add_restraints(const Topo::Rule rule, const Topo& topo,
                     cif::Loop& restr_loop, int (&counters)[5]) {
   //using gemmi::to_str;
   const auto& to_str = gemmi::to_str_prec<3>; // to make comparisons easier
@@ -296,7 +291,7 @@ void add_restraints(const Topo::Rule rule,
                         ".", ".", obs});
   } else if (rule.rkind == Topo::RKind::Chirality) {
     const Topo::Chirality& t = topo.chirs[rule.index];
-    double vol = rt.chiral_abs_volume(*t.restr);
+    double vol = topo.ideal_chiral_abs_volume(t);
     std::string obs = to_str3(t.calculate()) + " # " + t.atoms[0]->name +
                                                  " " + t.atoms[1]->name +
                                                  " " + t.atoms[2]->name +
@@ -325,7 +320,6 @@ void add_restraints(const Topo::Rule rule,
 }
 
 cif::Document make_rst(const Topo& topo, const gemmi::MonLib& monlib) {
-  using Provenance = Topo::Provenance;
   cif::Document doc;
   doc.blocks.emplace_back("restraints");
   cif::Block& block = doc.blocks[0];
@@ -337,25 +331,22 @@ cif::Document make_rst(const Topo& topo, const gemmi::MonLib& monlib) {
   for (const Topo::ChainInfo& chain_info : topo.chain_infos) {
     for (const Topo::ResInfo& ri : chain_info.res_infos) {
       // write link
-      for (const Topo::ResInfo::Prev& prev : ri.prev) {
-        const gemmi::Residue* prev_res = prev.get(&ri)->res;
-        const gemmi::ChemLink* link = monlib.find_link(prev.link);
-        if (link && count_provenance(ri.rules, Provenance::PrevLink) > 0) {
-          std::string comment = " link " + prev.link + " " +
-                                prev_res->seqid.str() + " " +
-                                prev_res->name + " - " +
+      for (const Topo::Link& prev : ri.prev) {
+        const gemmi::ChemLink* link = monlib.find_link(prev.link_id);
+        if (link && !prev.link_rules.empty()) {
+          std::string comment = " link " + prev.link_id + " " +
+                                prev.res1->seqid.str() + " " +
+                                prev.res1->name + " - " +
                                 ri.res->seqid.str() + " " + ri.res->name;
-          restr_loop.add_comment_and_row({comment, "LINK", ".",
-                                          cif::quote(prev.link), ".",
+          restr_loop.add_comment_and_row({comment, "LINK", ".", cif::quote(prev.link_id), ".",
                                           ".", ".", ".", ".", ".", ".", ".", ".", "."});
-          for (const Topo::Rule& rule : ri.rules)
-            if (rule.provenance == Provenance::PrevLink)
-              add_restraints(rule, topo, link->rt, restr_loop, counters);
+          for (const Topo::Rule& rule : prev.link_rules)
+            add_restraints(rule, topo, restr_loop, counters);
         }
       }
       // write monomer
-      if (count_provenance(ri.rules, Provenance::Monomer) > 0) {
-        std::string res_info = " monomer " + chain_info.name + " " +
+      if (!ri.monomer_rules.empty()) {
+        std::string res_info = " monomer " + chain_info.subchain_name + " " +
                                ri.res->seqid.str() + " " + ri.res->name;
         if (!ri.mods.empty())
           res_info += " modified by " + gemmi::join_str(ri.mods, ", ");
@@ -367,22 +358,20 @@ cif::Document make_rst(const Topo& topo, const gemmi::MonLib& monlib) {
 
         restr_loop.add_comment_and_row({res_info, "MONO", ".", group, ".",
                                         ".", ".", ".", ".", ".", ".", ".", ".", "."});
-        for (const Topo::Rule& rule : ri.rules)
-          if (rule.provenance == Provenance::Monomer)
-            add_restraints(rule, topo, ri.chemcomp.rt, restr_loop, counters);
+        for (const Topo::Rule& rule : ri.monomer_rules)
+          add_restraints(rule, topo, restr_loop, counters);
       }
     }
   }
   // explicit links
-  for (const Topo::ExtraLink& extra_link : topo.extras) {
-    const gemmi::ChemLink* chem_link = monlib.find_link(extra_link.link_id);
+  for (const Topo::Link& extra : topo.extras) {
+    const gemmi::ChemLink* chem_link = monlib.find_link(extra.link_id);
     assert(chem_link);
     std::string comment = " link " + chem_link->id;
-    restr_loop.add_comment_and_row({comment, "LINK", ".",
-                                    cif::quote(chem_link->id), ".",
+    restr_loop.add_comment_and_row({comment, "LINK", ".", cif::quote(chem_link->id), ".",
                                     ".", ".", ".", ".", ".", ".", ".", ".", "."});
-    for (const Topo::Rule& rule : extra_link.rules)
-      add_restraints(rule, topo, chem_link->rt, restr_loop, counters);
+    for (const Topo::Rule& rule : extra.link_rules)
+      add_restraints(rule, topo, restr_loop, counters);
   }
   return doc;
 }
@@ -402,9 +391,11 @@ int GEMMI_MAIN(int argc, char **argv) {
   }
   std::string input = p.coordinate_input_file(0);
   std::string output = p.nonOption(1);
+  bool verbose = p.options[Verbose];
   try {
-    gemmi::Structure st = gemmi::read_structure_gz(input,
-                                            gemmi::CoorFormat::UnknownAny);
+    if (verbose)
+      printf("Reading %s ...\n", input.c_str());
+    gemmi::Structure st = gemmi::read_structure_gz(input, gemmi::CoorFormat::Detect);
     if (st.input_format == gemmi::CoorFormat::Pdb ||
         st.input_format == gemmi::CoorFormat::ChemComp)
       gemmi::setup_entities(st);
@@ -413,18 +404,25 @@ int GEMMI_MAIN(int argc, char **argv) {
       return 1;
     }
     gemmi::Model& model0 = st.models[0];
+    if (verbose)
+      printf("Reading monomer library...\n");
     gemmi::MonLib monlib = gemmi::read_monomer_lib(monomer_dir,
                                                 model0.get_all_residue_names(),
                                                 gemmi::read_cif_gz);
 
     Topo topo;
+    topo.warnings = &std::cerr;
+    if (verbose)
+      printf("Preparing topology...\n");
     topo.initialize_refmac_topology(st, model0, monlib);
 
     // add H, sort atoms in residues and assign serial numbers
+    if (verbose)
+      printf("Hydrogens, sorting...\n");
     int serial = 0;
     for (Topo::ChainInfo& chain_info : topo.chain_infos)
       for (Topo::ResInfo& ri : chain_info.res_infos) {
-        const gemmi::ChemComp &cc = ri.chemcomp;
+        const gemmi::ChemComp& cc = ri.chemcomp;
         gemmi::Residue &res = *ri.res;
         if (!p.options[KeepHydrogens]) {
           gemmi::remove_hydrogens(res);
@@ -444,7 +442,7 @@ int GEMMI_MAIN(int argc, char **argv) {
                                                 : a.altloc < b.altloc;
                   });
         if (1) {  // temporary addition for makecif/refmac compatibility
-          if (gemmi::in_vector(std::string("AA-STAND"),  ri.mods) &&
+          if (gemmi::in_vector(std::string("AA-STAND"), ri.mods) &&
               !ri.res->find_atom("OXT", '*')) {
             gemmi::Atom atom;
             atom.name = "OXT";
@@ -454,31 +452,27 @@ int GEMMI_MAIN(int argc, char **argv) {
             atom.occ = 0;
             atom.b_iso = 0;
             ri.res->atoms.push_back(atom);
+            ri.chemcomp.atoms.push_back({atom.name, gemmi::El::O, 0.0, "OC"});
           }
         }
         for (gemmi::Atom& atom : res.atoms)
           atom.serial = ++serial;
       }
 
+    if (verbose)
+      printf("Preparing restraints...\n");
     topo.finalize_refmac_topology(monlib);
 
-    if (!p.options[KeepHydrogens] && !p.options[NoHydrogens])
-      for (Topo::ChainInfo& chain_info : topo.chain_infos)
-        for (Topo::ResInfo& ri : chain_info.res_infos)
-          for (gemmi::Atom& atom : ri.res->atoms)
-            if (!atom.is_hydrogen()) {
-              try {
-                place_hydrogens(atom, ri, topo);
-              } catch (const std::runtime_error& e) {
-                std::string loc = gemmi::atom_str(chain_info.name, *ri.res,
-                                                  atom.name, atom.altloc);
-                printf("Placing of hydrogen bonded to %s failed:\n  %s\n",
-                       loc.c_str(), e.what());
-              }
-            }
+    if (!p.options[KeepHydrogens] && !p.options[NoHydrogens]) {
+      if (verbose)
+        printf("Moving hydrogens to riding positions...\n");
+      place_hydrogens_on_all_atoms(topo);
+    }
 
-    cif::Document crd = make_crd(st, monlib, topo);
-    if (p.options[Verbose])
+    if (verbose)
+      printf("Preparing data for crd file...\n");
+    cif::Document crd = make_crd(st, topo);
+    if (verbose)
       printf("Writing coordinates to: %s.crd\n", output.c_str());
     {
       gemmi::Ofstream os(output + ".crd");
@@ -490,14 +484,16 @@ int GEMMI_MAIN(int argc, char **argv) {
         for (gemmi::Residue& res : chain.residues)
           for (gemmi::Atom& atom : res.atoms)
             if (atom.occ <= 0) {
-              if (p.options[Verbose])
+              if (verbose)
                 printf("Atom with zero occupancy: %s\n",
                        gemmi::atom_str(chain, res, atom).c_str());
               atom.name += '?';  // hide the atom by mangling the name
             }
 
+    if (verbose)
+      printf("Preparing data for rst file...\n");
     cif::Document rst = make_rst(topo, monlib);
-    if (p.options[Verbose])
+    if (verbose)
       printf("Writing restraints to: %s.rst\n", output.c_str());
     {
       gemmi::Ofstream os(output + ".rst");
