@@ -4,9 +4,9 @@
 #include <cstdlib>   // for getenv
 #include <stdexcept>
 #include <iostream>  // for cout
-#include <gemmi/chemcomp.hpp>  // for ChemComp
 #include <gemmi/mmcif.hpp>     // for make_structure
-#include <gemmi/polyheur.hpp>  // for remove_hydrogens, setup_entities
+#include <gemmi/polyheur.hpp>  // for setup_entities
+#include <gemmi/modify.hpp>    // for remove_hydrogens
 #include <gemmi/to_cif.hpp>    // for write_cif_to_file
 #include <gemmi/to_mmcif.hpp>  // for make_mmcif_document
 #include <gemmi/to_pdb.hpp>    // for write_pdb
@@ -105,10 +105,10 @@ int GEMMI_MAIN(int argc, char **argv) {
         std::printf("Reading %zu monomers and all links from %s\n",
                     res_names.size(), input.c_str());
       gemmi::MonLib monlib = gemmi::read_monomer_lib(monomer_dir, res_names,
-                                                     gemmi::read_cif_gz);
+                                                     gemmi::read_cif_gz, true);
       for (size_t i = 0; i != st.models.size(); ++i)
         // preparing topology modifies hydrogens in the model
-        prepare_topology(st, monlib, i, h_change, p.options[Sort]);
+        prepare_topology(st, monlib, i, h_change, p.options[Sort], &std::cerr);
     }
     if (p.options[Verbose])
       std::printf("Hydrogen site count: %zu in input, %zu in output.\n",
